@@ -912,21 +912,36 @@ where
     fn parse(&mut self) -> PResult<SupportsInParens> {
         let state = self.input.state();
 
-        expect!(self, "(");
+        match self.parse() {
+            Ok(feature) => Ok(SupportsInParens::Feature(feature)),
+            Err(_) => {
+                self.input.reset(&state);
 
-        self.input.skip_ws()?;
+                let mut parse_condition = || {
+                    expect!(self, "(");
 
-        if !is!(self, "(") && !is_case_insensitive_ident!(self, "not") {
-            self.input.reset(&state);
+                    let condition = self.parse()?;
 
-            return Ok(SupportsInParens::Feature(self.parse()?));
+                    expect!(self, ")");
+
+                    Ok(SupportsInParens::SupportsCondition(condition))
+                };
+
+                match parse_condition() {
+                    Ok(condition) => Ok(condition),
+                    Err(_) => {
+                        self.input.reset(&state);
+
+                        match self.parse() {
+                            Ok(general_enclosed) => {
+                                Ok(SupportsInParens::GeneralEnclosed(general_enclosed))
+                            }
+                            Err(err) => Err(err),
+                        }
+                    }
+                }
+            }
         }
-
-        let condition = SupportsInParens::SupportsCondition(self.parse()?);
-
-        expect!(self, ")");
-
-        Ok(condition)
     }
 }
 
@@ -935,17 +950,63 @@ where
     I: ParserInput,
 {
     fn parse(&mut self) -> PResult<SupportsFeature> {
-        expect!(self, "(");
+        match cur!(self) {
+            tok!("(") => {
+                bump!(self);
 
-        self.input.skip_ws()?;
+                self.input.skip_ws()?;
 
-        let declaration = self.parse()?;
+                let declaration = self.parse()?;
 
-        self.input.skip_ws()?;
+                self.input.skip_ws()?;
 
-        expect!(self, ")");
+                expect!(self, ")");
 
-        Ok(SupportsFeature::Declaration(declaration))
+                Ok(SupportsFeature::Declaration(declaration))
+            }
+            Token::Function { value, .. } if &*value.to_lowercase() == "selector" => {
+                let ctx = Ctx {
+                    in_supports_at_rule: true,
+                    ..self.ctx
+                };
+                let function = self.with_ctx(ctx).parse_as::<Function>()?;
+
+                Ok(SupportsFeature::Function(function))
+            }
+            _ => {
+                let span = self.input.cur_span()?;
+
+                Err(Error::new(
+                    span,
+                    ErrorKind::Expected("'(' or 'function' token"),
+                ))
+            }
+        }
+    }
+}
+
+impl<I> Parse<GeneralEnclosed> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<GeneralEnclosed> {
+        match cur!(self) {
+            tok!("function") => Ok(GeneralEnclosed::Function(self.parse()?)),
+            tok!("(") => {
+                bump!(self);
+
+                // TODO validate on first ident
+                Ok(GeneralEnclosed::SimpleBlock(self.parse_simple_block(')')?))
+            }
+            _ => {
+                let span = self.input.cur_span()?;
+
+                Err(Error::new(
+                    span,
+                    ErrorKind::Expected("function or '(' token"),
+                ))
+            }
+        }
     }
 }
 
@@ -1585,6 +1646,7 @@ where
         let ctx = Ctx {
             in_page_at_rule: true,
             grammar: Grammar::DeclarationList,
+            ..self.ctx
         };
         let block = self.with_ctx(ctx).parse_as::<SimpleBlock>()?;
 
